//ovladani zavor servo SG90, zvuk pres rele

//Jako inspiraci a vyuziti kodu, predevsim dcc, jsem pouzil toto https://sites.google.com/site/sidloweb/elektrika/22-uni16ard
//pouziti knihovny NmraDcc http://mrrwa.org/
//verze 4.3
//license GPLv2

#include <Servo.h>
#include <NmraDcc.h>

const uint8_t SW_VERSION = 43;
const uint16_t CV_AUXILIARY_ACTIVATION = 2;
const uint8_t VALUE_AUXILIARY_ACTIVATION = 9;   // change this value to restore CV defaults after upload sketch
const uint16_t CV_DECODER_KEY = 15;
const uint8_t VALUE_DECODER_KEY = 0;           // unlocked decoder
const uint16_t CV_DECODER_LOCK = 16;
const uint8_t VALUE_DECODER_LOCK = 0;          // unlocked decoder
const uint16_t CV_ROCO_ADDRESS = 34;
const uint8_t VALUE_ROCO_ADDRESS = 1;         // 1 - ROCO address, 0 - NMRA address
const uint16_t CV_PROD_ID_1 = 47;
const uint8_t VALUE_PROD_ID_1 = 1;              // productID #1 CZ
const uint16_t CV_PROD_ID_2 = 48;
const uint8_t VALUE_PROD_ID_2 = 29;              // productID #2
const uint16_t CV_PROD_ID_3 = 49;
const uint8_t VALUE_PROD_ID_3 = 4;              // productID #3
const uint16_t CV_PROD_ID_4 = 50;
const uint8_t VALUE_PROD_ID_4 = 3;             // productID #4

const uint8_t INIT_DECODER_ADDRESS = 98 ;      // ACCESSORY DECODER ADDRESS default
uint16_t thisDecoderAddress = 98 ;     //adresa zapnuti automatiky: 0 - bez, 1 - automat

const byte ACK_BUSY_PIN = 13;
byte automat = 1;

NmraDcc  Dcc;
DCC_MSG  Packet;

uint8_t lsb;
uint8_t msb;
uint8_t rocoAddress;
uint8_t decoderKey;
uint8_t decoderLock;

struct CVPair
{
  uint16_t  CV;
  uint8_t   Value;
};


CVPair FactoryDefaultCVs [] =
{
  {CV_ACCESSORY_DECODER_ADDRESS_LSB, INIT_DECODER_ADDRESS},
  {CV_ACCESSORY_DECODER_ADDRESS_MSB, 0},
  {CV_AUXILIARY_ACTIVATION, VALUE_AUXILIARY_ACTIVATION},
  {CV_DECODER_KEY, VALUE_DECODER_KEY},
  {CV_DECODER_LOCK, VALUE_DECODER_LOCK},
  {CV_ROCO_ADDRESS, VALUE_ROCO_ADDRESS},
  {CV_PROD_ID_1, VALUE_PROD_ID_1},
  {CV_PROD_ID_2, VALUE_PROD_ID_2},
  {CV_PROD_ID_3, VALUE_PROD_ID_3},
  {CV_PROD_ID_4, VALUE_PROD_ID_4},
  {129, 1},     // spozdeni IN senzor 0 lsb
  {130, 0},     // spozdeni IN senzor 0 msb
  {131, 136},     // spozdeni OUT senzor 0 lsb      //spozdeni CV131*256+CV130 = 5000 nebo (CV131 << 8) | CV130
  {132, 19},     // spozdeni OUT senzor 0 msb
  {133, 1},     // spozdeni IN senzor 1 lsb
  {134, 0},     // spozdeni IN senzor 1 msb
  {135, 232},     // spozdeni OUT senzor 1 lsb      //spozdeni CV134*256+CV133 = 1000
  {136, 3},       // spozdeni OUT senzor 1 msb
  {137, 1},     // spozdeni IN senzor 2 lsb
  {138, 0},     // spozdeni IN senzor 2 msb
  {139, 232},     // spozdeni OUT senzor 2 lsb      //spozdeni CV137*256+CV136 = 1000
  {140, 3},     // spozdeni OUT senzor 2 msb
  {141, 1},       // spozdeni INT senzor 3 lsb
  {142, 0},     // spozdeni IN senzor 3 msb
  {143, 136},     // spozdeni OUT senzor 3 lsb      //spozdeni CV140*256+CV139 = 5000
  {144, 19},        // spozdeni OUT senzor 3 msb
  {145, 135},     //pozice serva 1 otevrene zavory
  {146, 95},     //pozice serva 1 zavrene zavory
  {147, 20},       //rychlost pohybu serva 1
  {148, 25},      //pozice serva 2 otevrene zavory
  {149, 53},      //pozice serva 2 zavrene zavory
  {150, 20},      //rychlost pohybu serva 2
  {151, 48},      //reset lsb                       //doba k resetu CV148*256+CV147 = 30000
  {152, 117}      //reset msb
};

uint8_t FactoryDefaultCVSize = sizeof(FactoryDefaultCVs) / sizeof(CVPair);

//serva
Servo myServo1;
Servo myServo2;

const int Servo1_PIN = 7;     //E
const int Servo2_PIN = 6;     //F

byte srv1OP = 135;       //pozice serva 1 otevrene zavory
byte srv1CL = 95;       //pozice serva 1 zavrene zavory
byte srv1SP = 20;       //rychlost pohybu serva 1
byte srv1Pozice = 95;    //pozice serva 1
unsigned long srv1Cas;  //cas pro servo 1
byte srv2OP = 25;
byte srv2CL = 53;
byte srv2SP = 20;
byte srv2Pozice = 53;
unsigned long srv2Cas;

// senzor              0   1  2  3
//                     A   B  C  D
byte Senzor_PIN[] = { 19, 18, 9, 8};
unsigned int Senzor_IN[] =        { 1, 1, 1, 1};              	 //spozdeni akce pri prvnim najeti na cidlo
unsigned int Senzor_OUT[] =       { 5000, 1000, 1000, 5000};     //spozdeni akce po poslednim najeti na cidlo
struct Senzor
{
  byte  vstupPin;                 //pin pripojeni cidla
  byte  jednou;                   //1 - pouze najeti na cidlo, 0 - najeti i opusteni cidla
  byte  minule;
  byte  stav;                     //0 - neaktivni, 1 - najeti na cidlo po spozdeniIN, 2 - vyjezd z cidla po spozdeniOUT, 3 - oznaceni stavu mezi 1 a 2 (nastavuje se pouze ve vyhodnoceni)
  unsigned int spozdeniIN;
  unsigned int spozdeniOUT;
  unsigned long casMillis;
};
Senzor cidlo[4];        //pocet cidel

//vystupy LED
const int Led_B = 10;       //Bila I
const int Led_C1 = 11;      //Cervena 1 J
const int Led_C2 = 12;      //Cervena 2 K
const int Led_1B = 3;       //Bila L
const int Led_1C1 = 14;      //Cervena 1 M
const int Led_1C2 = 15;      //Cervena 2 N

//vystup rele
const int Rele1_PIN = 5;          //play/stop G, pri 12V rele O, pripojeni odporu 33k na ADKEY1
const int Rele2_PIN = 4;          //loop H, pri 12V rele P, pripojeni odporu 33k+18k na ADKEY1

unsigned int blikB = 665;           //interval bile 40/minuta
unsigned int blikC = 999;           //interval cervene 60/minuta
unsigned long CasReset = 30000;     //po uplynuti dojde k resetu jako pred vjezdem, pokud je 0, zadny reset se neprovede
unsigned long ImpulsZvuk = 50;      //delka impulsu pro spusteni zvuku
unsigned long CasZmenyBlik = 0;
unsigned long CasZvuk = 0;
unsigned long casMillisRes = 0;
int pocitadlo = 0;
int smer = 0;

boolean Sviti_B = true;     //blikani bile, pri startu blika
boolean Sviti_C = false;    //blikani cervena
boolean Svetla_B = false;
boolean Svetla_C = false;
boolean setDole = false;    //zavory dolu
boolean setDole1 = false;
boolean setDole2 = false;
boolean setNahore = true;   //zavory nahoru, pri startu zvednout
boolean setNahore1 = true;
boolean setNahore2 = true;
boolean zDole = false;      //stav zavor
boolean zNahore = false;
boolean setZvuk = false;    //prehrani zvuku

void setup() {
  pinMode(ACK_BUSY_PIN, OUTPUT);
  digitalWrite(ACK_BUSY_PIN, LOW);

  // Setup which External Interrupt, the Pin it's associated with that we're using and enable the Pull-Up
  Dcc.pin(0, 2, 1);

  // Call the main DCC Init function to enable the DCC Receiver
  Dcc.init( MAN_ID_DIY, SW_VERSION, FLAGS_OUTPUT_ADDRESS_MODE | FLAGS_DCC_ACCESSORY_DECODER, 0 );

  uint8_t cvVersion = Dcc.getCV(CV_AUXILIARY_ACTIVATION);
  if (cvVersion != VALUE_AUXILIARY_ACTIVATION) {
    setFactoryDefault();
  }


  for (int i = 0; i < 4; i++) {
    pinMode(Senzor_PIN[i], INPUT_PULLUP);
    cidlo[i].vstupPin = Senzor_PIN[i];
    cidlo[i].jednou = 0;
    cidlo[i].minule = 0;
    cidlo[i].stav = 0;
    cidlo[i].spozdeniIN = Senzor_IN[i];
    cidlo[i].spozdeniOUT = Senzor_OUT[i];
    cidlo[i].casMillis = 0;
  }
  pinMode(Led_B, OUTPUT);
  digitalWrite(Led_B, LOW);
  pinMode(Led_C1, OUTPUT);
  digitalWrite(Led_C1, LOW);
  pinMode(Led_C2, OUTPUT);
  digitalWrite(Led_C2, LOW);
  pinMode(Led_1B, OUTPUT);
  digitalWrite(Led_1B, LOW);
  pinMode(Led_1C1, OUTPUT);
  digitalWrite(Led_1C1, LOW);
  pinMode(Led_1C2, OUTPUT);
  digitalWrite(Led_1C2, LOW);

  delay(3000);
  pinMode(Rele1_PIN, OUTPUT);
  digitalWrite(Rele1_PIN, HIGH);
  pinMode(Rele2_PIN, OUTPUT);
  digitalWrite(Rele2_PIN, LOW);       //nastavi prehravani ve smycce, na karte musi byt pouze 1 zvuk
  delay(50);
  digitalWrite(Rele2_PIN, HIGH);      //necha se 2 sekundy hrat zvuk
  delay(2000);
  digitalWrite(Rele1_PIN, LOW);       //zastaveni prehravani
  delay(50);
  digitalWrite(Rele1_PIN, HIGH);

  initLocalVariables();

  myServo1.attach(Servo1_PIN);
  myServo2.attach(Servo2_PIN);

  srv1Cas = millis();
  srv2Cas = millis();
}


void loop() {
  // You MUST call the NmraDcc.process() method frequently from the Arduino loop() function for correct library operation
  Dcc.process();

  if (automat == 1) {
    stavS(pocitadlo);
    vyhodnoceniS();
  }
  Bila();
  Cervena();
  Dolu();
  Nahoru();
  Zvuk();

  pocitadlo++;
  if (pocitadlo >= 4) {
    pocitadlo = 0;
  }

}

void Bila () {
  if (Sviti_B) {
    digitalWrite(Led_C1, LOW);
    digitalWrite(Led_C2, LOW);
    digitalWrite(Led_B, Svetla_B);
    digitalWrite(Led_1C1, LOW);
    digitalWrite(Led_1C2, LOW);
    digitalWrite(Led_1B, Svetla_B);
    if ((millis() - CasZmenyBlik) > blikB)
    {
      CasZmenyBlik = millis();
      Svetla_B = !Svetla_B;
    }
  }
}

void Cervena () {
  if (Sviti_C) {
    digitalWrite(Led_B, LOW);
    digitalWrite(Led_C1, Svetla_C);
    digitalWrite(Led_C2, !Svetla_C);
    digitalWrite(Led_1B, LOW);
    digitalWrite(Led_1C1, Svetla_C);
    digitalWrite(Led_1C2, !Svetla_C);
    //Zvuk(!Svetla_C);
    if ((millis() - CasZmenyBlik) > blikC)
    {
      CasZmenyBlik = millis();
      Svetla_C = !Svetla_C;
      //Zvuk(Svetla_C);
    }
  }
}

void Tma () {
  digitalWrite(Led_C1, LOW);
  digitalWrite(Led_C2, LOW);
  digitalWrite(Led_B, LOW);
  digitalWrite(Led_1C1, LOW);
  digitalWrite(Led_1C2, LOW);
  digitalWrite(Led_1B, LOW);
}

void Zvuk () {
  if (setZvuk) {
    if ((unsigned long) (millis() - CasZvuk) < ImpulsZvuk) {      //sepne prehravani ve smycce
      digitalWrite(Rele2_PIN, LOW);
    } else {
      digitalWrite(Rele2_PIN, HIGH);
    }
  } else {
    if ((unsigned long) (millis() - CasZvuk) < ImpulsZvuk) {       //sepne -> bude start nebo stop
      digitalWrite(Rele1_PIN, LOW);
    } else {
      digitalWrite(Rele1_PIN, HIGH);
    }
  }
}

void Nahoru() {
  if (setNahore) {
    zDole = false;
    if (setNahore1) {
      Nahoru1();
    }
    if (setNahore2) {
      Nahoru2();
    }
  }
}

void Nahoru1() {
  unsigned long currentMillis;
  currentMillis = millis();
  if ((unsigned long) (currentMillis - srv1Cas) < srv1SP) {
    //nic
    return;
  }

  srv1Pozice++;
  myServo1.write(srv1Pozice);
  srv1Cas = currentMillis;
  if (srv1Pozice >= srv1OP) {
    setNahore1 = false;
    setNahore = setNahore1 | setNahore2;
    zNahore = true;
  }
}

void Nahoru2() {
  unsigned long currentMillis;
  currentMillis = millis();
  if ((unsigned long) (currentMillis - srv2Cas) < srv2SP) {
    //nic
    return;
  }

  srv2Pozice--;
  myServo2.write(srv2Pozice);
  srv2Cas = currentMillis;
  if (srv2Pozice <= srv2OP) {
    setNahore2 = false;
    setNahore = setNahore1 | setNahore2;
    zNahore = true;
  }
}

void Dolu() {
  if (setDole) {
    zNahore = false;
    if (setDole1) {
      Dolu1();
    }
    if (setDole2) {
      Dolu2();
    }
    if (zDole && automat == 1) {
      CasZvuk = millis();
      setZvuk = false;
    }
  }
}



void Dolu1() {
  unsigned long currentMillis;
  currentMillis = millis();
  if ((unsigned long) (currentMillis - srv1Cas) < srv1SP) {
    //nic
    return;
  }

  srv1Pozice--;
  myServo1.write(srv1Pozice);
  srv1Cas = currentMillis;
  if (srv1Pozice <= srv1CL) {
    setDole1 = false;
    setDole = setDole1 | setDole2;
    zDole = true;
  }
}

void Dolu2() {
  unsigned long currentMillis;
  currentMillis = millis();
  if ((unsigned long) (currentMillis - srv2Cas) < srv2SP) {
    //nic
    return;
  }

  srv2Pozice++;
  myServo2.write(srv2Pozice);
  srv2Cas = currentMillis;
  if (srv2Pozice >= srv2CL) {
    setDole2 = false;
    setDole = setDole1 | setDole2;
    zDole = true;
  }
}

//zjistovani stavu senzoru(cidel)
void stavS (int index) {
  unsigned long currentMillis;
  currentMillis = millis();

  if (cidlo[index].stav == 0) {
    if ((cidlo[index].minule == 0) && (!digitalRead(cidlo[index].vstupPin))) {           //najeti na cidlo, plati pro vsechna cidla
      cidlo[index].minule = 1;                                                           //oznaceni prvniho najeti
      cidlo[index].casMillis = currentMillis;                                            //cas prvniho najeti na cidlo
      casMillisRes = millis();                                                           //nastaveni casu pro pripadny reset
    }
    if (cidlo[index].minule == 1) {
      if ((unsigned long) (currentMillis - cidlo[index].casMillis) < cidlo[index].spozdeniIN) {
        //nic
      } else {
        cidlo[index].stav = 1;                                                          //najeti na cidlo a uplynuti spozdeniIN
        cidlo[index].minule = 2;                                                        //oznaceni uplynuti spozdeniIN
      }
    }
  } else if ((cidlo[index].stav == 1) || (cidlo[index].stav == 3)) {                 //muze nastat az po najeti na cidlo a uplynuti spozdeniIN, pripadne po nastaveni na stav 3
    if (cidlo[index].jednou == 0) {                                                  //testovat i vyjezd z cidla
      if (!digitalRead(cidlo[index].vstupPin)) {                                     //pri kazdem najeti na cidlo se cas vynuluje, pokud se najeti vejde do intervalu spozdeniOUT
        cidlo[index].casMillis = currentMillis;
        casMillisRes = millis();                                                     //nastaveni casu pro pripadny reset
      } else {
        if ((unsigned long) (currentMillis - cidlo[index].casMillis) < cidlo[index].spozdeniOUT) {
          //nic
        } else {
          cidlo[index].stav = 2;                                                      //opusteni cidla a uplynuti spozdeniOUT
          cidlo[index].casMillis = currentMillis;                                     //cas stavu 2
        }
      }
    }
  }
}

//vyhodnocovani cidel
void vyhodnoceniS () {
  if (smer == 0) {
    if ((cidlo[3].minule == 0) && (cidlo[0].minule >= 1)) {                                  //nastaveni pro obousmerny provoz, prvni dve cidla podle smeru budou reagovat jenom na najeti
      cidlo[0].jednou = 1;
      cidlo[1].jednou = 1;
      smer = 1;
    }
    if ((cidlo[0].minule == 0) && (cidlo[3].minule >= 1)) {
      cidlo[3].jednou = 1;
      cidlo[2].jednou = 1;
      smer = 2;
    }
  }
  // Serial.print("smer "); Serial.println(smer);
  if (smer == 1) {
    if ((cidlo[1].stav == 1) && (zNahore)) {         //vjezd na prostredni cidlo, zavory nahore. spusteni zavor dolu a vypnuti zvuku, dal blikani cervene
      if ((!setDole) && (zNahore)) {
        srv1Cas = millis();
        srv2Cas = millis();
        cidlo[1].stav = 3;
        setDole = true;
        setDole1 = true;
        setDole2 = true;
      }
    }
    if ((cidlo[3].stav == 1) && (zDole)) {          //vjezd na krajni cidlo, zavory dole. zvednuti zavor nahoru a tma
      if ((!setNahore) && (zDole)) {
        srv1Cas = millis();
        srv2Cas = millis();
        cidlo[3].stav = 3;
        Sviti_C = false;
        Sviti_B = false;
        setNahore = true;
        setNahore1 = true;
        setNahore2 = true;
        Tma();
      }
    }
  } else if (smer == 2) {
    if ((cidlo[2].stav == 1) && (zNahore)) {         //vjezd na prostredni cidlo, zavory nahore. spusteni zavor dolu a vypnuti zvuku, dal blikani cervene
      if ((!setDole) && (zNahore)) {
        srv1Cas = millis();
        srv2Cas = millis();
        cidlo[2].stav = 3;
        setDole = true;
        setDole1 = true;
        setDole2 = true;
      }
    }  else if ((cidlo[0].stav == 1) && (zDole)) {    //vjezd na krajni cidlo, zavory dole. zvednuti zavor nahoru a tma
      if ((!setNahore) && (zDole)) {
        srv1Cas = millis();
        srv2Cas = millis();
        cidlo[0].stav = 3;
        Sviti_C = false;
        Sviti_B = false;
        setNahore = true;
        setNahore1 = true;
        setNahore2 = true;
        Tma();
      }
    }
  }
  if (((cidlo[3].stav == 2) || (cidlo[0].stav == 2)) || (((unsigned long) (millis() - casMillisRes) > CasReset) && (casMillisRes > 0) && (CasReset > 0))) {      //opusteni cidla, nesviti cervena a bila. zacne blikat bila, vse se vynuluje. Stejne tak po uplynuti doby pro celkovy reset od prvniho najeti na cidlo
    Sviti_C = false;
    Sviti_B = true;
    smer = 0;
    casMillisRes = 0;
    for (int i = 0; i < 4; i++) {
      cidlo[i].minule = 0;
      cidlo[i].stav = 0;
      cidlo[i].jednou = 0;
      cidlo[i].casMillis = 0;
    }
    if (setZvuk) {
      CasZvuk = millis();
      setZvuk = false;
    }
    if (zDole) {
      setNahore = true;
      setNahore1 = true;
      setNahore2 = true;
      srv1Cas = millis();
      srv2Cas = millis();
    }
  }
  if (((cidlo[3].stav == 1) || (cidlo[0].stav == 1)) && (Sviti_B)) {                        //vjezd na prvni cidlo, spusti se zvuk a blika cervena
    Sviti_C = true;
    Sviti_B = false;
    CasZvuk = millis();
    setZvuk = true;
  }
}

//DCC sekce
/*********************************************************************************
    potvrzeni zapisu
 */
void notifyCVAck() {
  digitalWrite(ACK_BUSY_PIN, HIGH) ;
  delay(6) ;
  digitalWrite(ACK_BUSY_PIN, LOW) ;
}

/*********************************************************************************
   Set factory default.
*/
void setFactoryDefault() {
  if (Dcc.isSetCVReady()) {
    for (uint16_t i = 0; i < FactoryDefaultCVSize; i++) {
      Dcc.setCV(FactoryDefaultCVs[i].CV, FactoryDefaultCVs[i].Value);
    }
  }
}

/**********************************************************************************
   Init local variables.
*/
void initLocalVariables() {
  lsb = Dcc.getCV(CV_ACCESSORY_DECODER_ADDRESS_LSB);
  msb = Dcc.getCV(CV_ACCESSORY_DECODER_ADDRESS_MSB);
  thisDecoderAddress = (msb << 8) | lsb;

  decoderKey = Dcc.getCV(CV_DECODER_KEY);
  decoderLock = Dcc.getCV(CV_DECODER_LOCK);

  rocoAddress = Dcc.getCV(CV_ROCO_ADDRESS);

  uint8_t val1;
  uint8_t val2;

  val1 = Dcc.getCV(129);
  val2 = Dcc.getCV(130);
  cidlo[0].spozdeniIN = (val2 << 8) | val1;
  val1 = Dcc.getCV(131);
  val2 = Dcc.getCV(132);
  cidlo[0].spozdeniOUT = (val2 << 8) | val1;
  val1 = Dcc.getCV(133);
  val2 = Dcc.getCV(134);
  cidlo[1].spozdeniIN = (val2 << 8) | val1;
  val1 = Dcc.getCV(135);
  val2 = Dcc.getCV(136);
  cidlo[1].spozdeniOUT = (val2 << 8) | val1;
  val1 = Dcc.getCV(137);
  val2 = Dcc.getCV(138);
  cidlo[2].spozdeniIN = (val2 << 8) | val1;
  val1 = Dcc.getCV(139);
  val2 = Dcc.getCV(140);
  cidlo[2].spozdeniOUT = (val2 << 8) | val1;
  val1 = Dcc.getCV(141);
  val2 = Dcc.getCV(142);
  cidlo[3].spozdeniIN = (val2 << 8) | val1;
  val1 = Dcc.getCV(143);
  val2 = Dcc.getCV(144);
  cidlo[3].spozdeniOUT = (val2 << 8) | val1;

  srv1OP = Dcc.getCV(145);
  srv1CL = Dcc.getCV(146);
  srv1SP = Dcc.getCV(147);
  srv2OP = Dcc.getCV(148);
  srv2CL = Dcc.getCV(149);
  srv2SP = Dcc.getCV(150);

  val1 = Dcc.getCV(151);
  val2 = Dcc.getCV(152);

  CasReset = (val2 << 8) | val1;
  srv1Pozice = srv1CL;
  srv2Pozice = srv2CL;
}

/**************************************************************************
   CV was changed.
*/
void notifyCVChange(uint16_t CV, uint8_t Value) {

  if (CV == CV_ACCESSORY_DECODER_ADDRESS_LSB) {
    lsb = Value;
    thisDecoderAddress = (msb << 8) | lsb;
    return;
  }

  if (CV == CV_ACCESSORY_DECODER_ADDRESS_MSB) {
    msb = Value;
    thisDecoderAddress = (msb << 8) | lsb;
    return;
  }

  if (CV == CV_ROCO_ADDRESS) {
    rocoAddress = Value;
    return;
  }

  if (CV == CV_DECODER_KEY) {
    decoderKey = Value;
    return;
  }

  if (CV == CV_DECODER_LOCK) {
    decoderLock = Value;
    return;
  }

  uint8_t val1;
  uint8_t val2;

  switch (CV) {
    case 129:
      val1 = Value;
      val2 = Dcc.getCV(130);
      cidlo[0].spozdeniIN = (val2 << 8) | val1;
      break;
    case 130:
      val1 = Dcc.getCV(129);
      val2 = Value;
      cidlo[0].spozdeniIN = (val2 << 8) | val1;
      break;
    case 131:
      val1 = Value;
      val2 = Dcc.getCV(132);
      cidlo[0].spozdeniOUT = (val2 << 8) | val1;
      break;
    case 132:
      val1 = Dcc.getCV(131);
      val2 = Value;
      cidlo[0].spozdeniOUT = (val2 << 8) | val1;
      break;
    case 133:
      val1 = Value;
      val2 = Dcc.getCV(134);
      cidlo[1].spozdeniIN = Value;
      break;
    case 134:
      val1 = Dcc.getCV(133);
      val2 = Value;
      cidlo[1].spozdeniIN = (val2 << 8) | val1;
      break;
    case 135:
      val1 = Value;
      val2 = Dcc.getCV(136);
      cidlo[1].spozdeniOUT = (val2 << 8) | val1;
      break;
    case 136:
      val1 = Dcc.getCV(135);
      val2 = Value;
      cidlo[1].spozdeniOUT = (val2 << 8) | val1;
      break;
    case 137:
      val1 = Value;
      val2 = Dcc.getCV(138);
      cidlo[2].spozdeniIN = Value;
      break;
    case 138:
      val1 = Dcc.getCV(137);
      val2 = Value;
      cidlo[2].spozdeniIN = (val2 << 8) | val1;
      break;
    case 139:
      val1 = Value;
      val2 = Dcc.getCV(140);
      cidlo[2].spozdeniOUT = (val2 << 8) | val1;
      break;
    case 140:
      val1 = Dcc.getCV(139);
      val2 = Value;
      cidlo[2].spozdeniOUT = (val2 << 8) | val1;
      break;
    case 141:
      val1 = Value;
      val2 = Dcc.getCV(142);
      cidlo[3].spozdeniIN = Value;
      break;
    case 142:
      val1 = Dcc.getCV(141);
      val2 = Value;
      cidlo[3].spozdeniIN = (val2 << 8) | val1;
      break;
    case 143:
      val1 = Value;
      val2 = Dcc.getCV(144);
      cidlo[3].spozdeniOUT = (val2 << 8) | val1;
      break;
    case 144:
      val1 = Dcc.getCV(143);
      val2 = Value;
      cidlo[3].spozdeniOUT = (val2 << 8) | val1;
      break;
    case 145:
      srv1OP = Value;
      break;
    case 146:
      srv1CL = Value;
      break;
    case 147:
      srv1SP = Value;
      break;
    case 148:
      srv2OP = Value;
      break;
    case 149:
      srv2CL = Value;
      break;
    case 150:
      srv2SP = Value;
      break;
    case 151:
      val1 = Value;
      val2 = Dcc.getCV(152);
      CasReset = (val2 << 8) | val1;
      break;
    case 152:
      val1 = Dcc.getCV(151);
      val2 = Value;
      CasReset = (val2 << 8) | val1;
      break;
  }
}
/********************************************************************
   Test for unlocked key.
*/
uint8_t isUnlocked(uint16_t CV) {

  if (CV == CV_DECODER_KEY) {
    return true;
  }

  if (decoderKey == 255) {
    return true;
  }

  if (decoderLock == decoderKey) {
    return true;
  }

  return false;
}

/******************************************************************************
   Check CV valid.
*/
uint8_t notifyCVValid(uint16_t CV, uint8_t Writable) {

  uint8_t unlocked = isUnlocked(CV);

  if (unlocked && Writable && (CV == CV_MANUFACTURER_ID))
    setFactoryDefault();

  if (!unlocked) {
    return false;
  }

  if (CV > MAXCV) {
    return false;
  }

  if (unlocked && Writable && (CV == CV_VERSION_ID)) {
    return false;
  }

  return true;
}


/**************************************************************************
   Backward compatibility 1.x.
*/
void notifyDccAccState( uint16_t Addr, uint16_t BoardAddr, uint8_t OutputAddr, uint8_t State) {

  uint8_t bitState = OutputAddr & 0x01;

  notifyDccAccTurnoutOutput(Addr, bitState, 1);

}

/**************************************************************************
   Since library NmraDcc 2.x
*/
void notifyDccAccTurnoutOutput( uint16_t Addr, uint8_t Direction, uint8_t OutputPower) {

  if (rocoAddress) {
    Addr = Addr + 4;
  }

  if ((Addr >= thisDecoderAddress)
      && (Addr <= thisDecoderAddress + 5)) {
    byte offAdr = Addr - thisDecoderAddress;
    switch (offAdr) {
      case 0:          //ovladani senzory (automatika) nebo se vypne vyhodnoceni a sledovani stavu
        automat = Direction;
        break;
      case 1:                           //blika bila
        Sviti_B = Direction;
        break;
      case 2:                           //blika cervena
        Sviti_C = Direction;
        break;
      case 3:                           //zavory nahoru, pokud jsou dole
        if (zDole) {
          setNahore = Direction;
          setNahore1 = Direction;
          setNahore2 = Direction;
          srv1Cas = millis();
          srv2Cas = millis();
        }
        break;
      case 4:                           //zavory dolu, pokud jsou nahore
        if (zNahore) {
          setDole = Direction;
          setDole1 = Direction;
          setDole2 = Direction;
          srv1Cas = millis();
          srv2Cas = millis();
        }
        break;
      case 5:                           //zvuk
        setZvuk = Direction;
        CasZvuk = millis();
        break;
    }
  }
}
